package koeln.uni.idh.java1.session11.zoo.animals;

import koeln.uni.idh.java1.session11.zoo.IsDrawable;
import koeln.uni.idh.java1.session11.zoo.Util;
import koeln.uni.idh.java1.session11.zoo.Zoo;

public abstract class Animal implements IsDrawable {
	static char[] directions = new char[] {'l', 't', 'r', 'b'};
	
	String name;

	int[] position;
	
	Zoo zoo;

	public abstract Animal reproduce(Animal partner);

	public abstract int getStepSize();
	
	public abstract char getSymbol();

	public abstract boolean canReproduce(Animal other);
	

	int direction;

	public Animal(Zoo zoo, String name, int[] initialPosition) {
		this.direction = Util.randInt(0, directions.length);
		this.zoo = zoo;
		this.name = name;
		this.position = initialPosition;
		System.out.println(this.getClass().getSimpleName() + " " + name + " is born at (" +this.position[0]+","+this.position[1]+ "). It's looking towards " + directions[direction] + ".");

	}

	public int getX() {
		return position[0];
	}

	public int getY() {
		return position[1];
	}

	
	public void tick() {
		int[] newPosition = this.position.clone();
		
		switch(direction) {
		case '0':
			newPosition[0] = newPosition[0] - getStepSize();
			break;
		case '1':
			newPosition[1] = newPosition[1] - getStepSize();
			break;
		case '2':
			newPosition[0] = newPosition[0] + getStepSize();
			break;
		case '3':
			newPosition[1] = newPosition[1] + getStepSize();
			break;
		}
		
		if (zoo.isFree(newPosition)) {
			this.position = newPosition;
			System.out.println("Animal " + name + " has moved to (" + this.position[0]+","+this.position[1]+")");
		} else {
			Animal potentialPartner = zoo.getAnimalByPosition(newPosition);
			if (potentialPartner != null && potentialPartner != this && canReproduce(potentialPartner)) {
				// Paarung
				Animal child = reproduce(potentialPartner);
				zoo.addAnimal(child);
				System.out.println("The animal " + child.name + " is born.");
			} else {
				// Richtung ändern
				turn(Util.randInt(-1, 1));
				System.out.println("The animal " + name + " has turned.");
				
			}
		}
	}
	
	public void turn(int d) {
		direction = (direction + 4 + (int)Math.signum(d))%4;
	}
	
	protected int[] getFreePositionThatIsClose() {
		for (int xd = -1; xd < 2; xd++) {
			for (int yd = -1; yd < 2; yd++) {
				if (xd == 0 && yd == 0)
					continue;
				int[] ppos = this.position.clone();
				ppos[0] = ppos[0] + xd;
				ppos[1] = ppos[1] + yd;
				if (zoo.isFree(ppos)) {
					return ppos;
				}
			}
		}
		return zoo.getRandomFreePosition();
	}

}
